/* global document */
import * as React from 'react';
import { createPortal } from 'react-dom';
import { useEffect, useMemo, useRef, useContext } from 'react';
import mapboxgl from '../utils/mapboxgl';
import { MapContext } from './map';
import { deepEqual } from '../utils/deep-equal';
// Adapted from https://github.com/mapbox/mapbox-gl-js/blob/v1.13.0/src/ui/popup.js
function getClassList(className) {
    return new Set(className ? className.trim().split(/\s+/) : []);
}
function Popup(props) {
    const map = useContext(MapContext);
    const container = useMemo(() => {
        return document.createElement('div');
    }, []);
    const popup = useMemo(() => {
        const options = { ...props };
        return new mapboxgl.Popup(options).setLngLat([props.longitude, props.latitude]);
    }, []);
    const thisRef = useRef({ props });
    thisRef.current.props = props;
    useEffect(() => {
        popup.on('open', e => {
            thisRef.current.props.onOpen?.(e);
        });
        popup.on('close', e => {
            thisRef.current.props.onClose?.(e);
        });
        popup.setDOMContent(container).addTo(map);
        return () => {
            popup.remove();
        };
    }, []);
    if (popup.isOpen()) {
        if (popup.getLngLat().lng !== props.longitude || popup.getLngLat().lat !== props.latitude) {
            popup.setLngLat([props.longitude, props.latitude]);
        }
        // @ts-ignore
        if (props.offset && !deepEqual(popup.options.offset, props.offset)) {
            popup.setOffset(props.offset);
        }
        // @ts-ignore
        if (popup.options.anchor !== props.anchor || popup.options.maxWidth !== props.maxWidth) {
            // @ts-ignore
            popup.options.anchor = props.anchor;
            popup.setMaxWidth(props.maxWidth);
        }
        // @ts-ignore
        if (popup.options.className !== props.className) {
            // @ts-ignore
            const prevClassList = getClassList(popup.options.className);
            const nextClassList = getClassList(props.className);
            for (const c of prevClassList) {
                if (!nextClassList.has(c)) {
                    popup.removeClassName(c);
                }
            }
            for (const c of nextClassList) {
                if (!prevClassList.has(c)) {
                    popup.addClassName(c);
                }
            }
            // @ts-ignore
            popup.options.className = props.className;
        }
    }
    return createPortal(props.children, container);
}
// @ts-ignore
export default React.memo(Popup);
//# sourceMappingURL=popup.js.map